/*
 * Copyright (c) 2003, 2004 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-xtextify.c,v 1.5 2014/02/06 00:28:04 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/test.h"
#include "sm/str2rcb.h"
#include "sm/io.h"

#define SMAXLEN	64	/* don't change, see below: "too long" */
#define SNORLEN	32

#define XISOK SM_TEST(strncmp(b, (char *) sm_str_data(s), sm_str_getlen(s)) == 0)

static void
test_harness(void)
{
	sm_str_P s;
	sm_ret_T ret;
	char *b;

	s = sm_str_new(NULL, SNORLEN, SMAXLEN);
	SM_TEST(s != NULL);

	b = "abc";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	XISOK;

	b = "abcdefghijklmnopqrstuvwxyz";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	XISOK;

	b = "abc+";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B";
	XISOK;

	b = "abc\\";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+5C";
	XISOK;

	b = "abc+\\";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5C";
	XISOK;

	b = "abc+\\abc(";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5Cabc+28";
	XISOK;

	b = "abc+\\abc()";
	sm_str_clr(s);
	ret = xtextify(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5Cabc+28)";
	XISOK;

	b = "abc+\\abc()";
	sm_str_clr(s);
	ret = xtextify(b, ")", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5Cabc+28+29";
	XISOK;

	b = "abc+\\abc()<>";
	sm_str_clr(s);
	ret = xtextify(b, ")", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5Cabc+28+29<>";
	XISOK;

	b = "abc+\\abc()<>";
	sm_str_clr(s);
	ret = xtextify(b, ")<>", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2B+5Cabc+28+29+3C+3E";
	XISOK;

	b = "0123456789012345678901234567890123456789abc+\\abc()<>";
	sm_str_clr(s);
	ret = xtextify(b, ")<>", s);
	SM_TEST(sm_is_err(ret));	/* result too long */

	sm_str_free(s);
}

static void
test2(void)
{
	sm_str_P s;
	sm_ret_T ret;
	char *b;

	s = sm_str_new(NULL, 4, 8);
	SM_TEST(s != NULL);

	b = "abc";
	sm_str_clr(s);
	ret = xtextify2(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	XISOK;

	b = "abc+de";
	sm_str_clr(s);
	ret = xtextify2(b, "", s);
	SM_TEST(ret == SM_SUCCESS);
	b = "abc+2Bde";
	XISOK;

	b = "abc+\\abc()";
	sm_str_clr(s);
	ret = xtextify2(b, "", s);
	SM_TEST(ret != SM_SUCCESS);
	b = "abc+2B";
	XISOK;
sm_io_fprintf(smioerr, "b=%s, s=%S\n", b, s);

	sm_str_free(s);
}

int
main(int argc, char *argv[])
{
	sm_test_begin(argc, argv, "test xtextify");
	test_harness();
	test2();
	return sm_test_end();
}
