#!/bin/sh
# $Id: t-rvrs-rel-0.sh,v 1.2 2014/07/07 19:53:31 ca Exp $
#
# ----------------------------------------
# test relaying for IP without PTR record
# uses smtpcnf.sh.
# ----------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_DNS_TEST is set"
  exit 0
fi
# this test requires DNS entries for 10.0.255.*
if test X"${MTA_HAVE_10_INADDRARPA}" != "Xtrue"
then
  echo "SKIPPED: $0: MTA_HAVE_10_INADDRARPA is not set to true"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps SS_TEST
chkcto smtps MTA_USE_TLS
srchastls

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create restrictions
${CAT} > ${ACC} <<EOF
cltresolve:no	error:450 4.7.1 client IP address currently cannot be checked via DNS properly -- maybe misconfigured
EOF
# create map
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < ${ACC}

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 16; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 12;
flags = {access, delay_checks};
tls { flags = { allow_relaying_if_verified } };
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
relay_from = "No.host.";
relay_to = "No.host.";
}
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

${CAT} > ${MT} <<EOF
local.host	lmtp:
other.host	[127.0.0.1]
meta1.org	[127.0.0.1]
EOF

# run some test programs

# ----------------------------------------
# allowing relaying via STARTTLS, overrides missing PTR record
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender${TEST}@meta1.org -Ruser@local.host -E 0 -X 0 -A 1 -V "-c 10.0.255.9" -Z smtpc2 -c "-@S"
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi
fi


# ----------------------------------------
rm -f ${ACCMAP} ${ALIMAP} ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
