#!/bin/sh
# $Id: t-schedtmo-0.sh,v 1.4 2014/01/02 21:15:23 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test handling of sinks that don't accept mail.
# ----------------------------------------
#

if test "X"${MTA_TIMING} = "X"
then
  echo "SKIPPED: $0: MTA_TIMING is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

# see below "AQ is getting full": that is prevented by FFR_AVLSLOTS
if hascto ./qmgr FFR_AVLSLOTS
then
  echo "SKIPPED: $0: test incompatible with FFR_AVLSLOTS"
  exit 0
fi

OUT="t-schedtmo-0.out"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create directory to pass socket
test -d smtps || mkdir smtps

rm -f ${QMGRCONFMAP} ${MTMAP} ${OUT}

# ----------------------------------------
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 14; pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
	connect_only_to = 127.0.0.1;
	io_timeout=60s;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
	retry_delay_min = 5s;
	retry_delay_max = 1m;
	delivery_timeout=30s;
	AQ_entries_max=32;
	scheduler_timeout=10;
	smtpc { initial_connections=1; connections_max=2; }
	start_action = wait; user = ${USER};
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF} -d 3";
}
smar { log_level = 14;
	start_action = wait; user = ${USER};
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

${CAT} > ${MT} <<EOF
dom1.tld	[10.1.1.1]
dom2.tld	[10.1.1.2]
EOF

# ----------------------------------------
# send some messages, sink does not accept any, AQ is getting full and
# entries should be written to defedb.
# make sure that ibdb and defedb have all rcpts that were sent.
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
clearlogs

if ${SHELL} ${SD}/mcpmts.sh -w 4 -Y '-A 451' -c '-q 50 -s 100 -m 50 -Y dom2.tld -F dom1.tld -xx -4 120 -3' -N -A "" -E "" -X 0 -Z smtpc2 -1
then
  QL=qmgr.log
  S2L=`grep -c sched2long $QL`
  if test $S2L -le 10
  then
    $VERBOSE && echo "$0: test ${TEST}: expected sched2long in $QL"
    ERR=true
  fi
  RCPTS=0
  ${EDBR} > ${OUT}
  DR=`grep -c 'got rec' ${OUT}`
  ${IDBR0} > ${OUT}
  IR=`grep -c 'RCPT: OPEN' ${OUT}`
  RCPTS=`expr $DR + $IR `
  SENT=`grep "total=" ${CLTL} | sed -e 's/^.*total=//' -e 's/ .*//'`
  if test X"${RCPTS}" != X"${SENT}"
  then
    $VERBOSE && echo "$0: test ${TEST}: ${RCPTS} != ${SENT}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${MTMAP} ${OUT}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
