%{

#include "sm/generic.h"
SM_RCSID("@(#)$Id: cond.y,v 1.22 2010/07/06 21:25:37 ca Exp $")

#include "sm/limits.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/memops.h"
#include "sm/string.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-expr.h"

#include "sm-expr.h"
#include "sm-conf-token.h"

#include "sm/io.h"
%}

%union {
	int			 v_op;
	long		 v_num;
	id_T		 v_id;
	uint		 v_exp;
}

%{

static void yyerror(void *ctx, char *msg)
{
	fprintf(stderr, "%s\n", msg);
	exit(1);
}

/*
uses a symbol table to look up the ids;
that symbol table should be part of the context passed to yy*().
that way it can be specific for each expression.
*/
extern sm_func_T functab[];

#define M_NAME_IS_ID	0x01
#define M_NAME_IS_FUNC	0x02

int YY_Verbose = 0;

static val_type_T
chk_name(lpctx_P lpctx, const char *id, char **endptr, uint *idx)
{
	size_t l;
	int i;
	char *e;

	assert(id != NULL);
	assert(endptr != NULL);
	assert(*endptr != NULL);
	e = (char *)id;
	while ((isalnum(*e) || *e == '_') && id < *endptr)
		++e;
	*endptr = e;
	l = e - id;
	i = e_bsrchn(lpctx->lpc_idtab, lpctx->lpc_idsz, id, l);
	if (i >= 0) {
if (YY_Verbose > 0)
sm_io_fprintf(smioerr, "chk_name: id=\"%s\", i=%d, idv=%d, vt=%d\n", id, i, lpctx->lpc_idtab[i].ide_idv, lpctx->lpc_idtab[i].ide_vt);
		if (idx != NULL)
			*idx = lpctx->lpc_idtab[i].ide_idv;
		return lpctx->lpc_idtab[i].ide_vt;
	}
	i = f_binsrchn(id, l);
	if (i >= 0) {
		if (idx != NULL)
			*idx = (uint) i;
		return vt_func;
	}
	return vt_none;
}

int yylex(YYSTYPE *LVALP, void *ctx);

%}

%pure-parser
%parse-param {void *ctx}
%lex-param {void *ctx}

%token CONST_INT
%token ID
%token ID_BOOL
%token ID_STRING
%token ID_INT
%token STR
%token COMMA
%token O_PARENTHESIS
%token C_PARENTHESIS
%token F_MATCH
%token F_STRCASECMP
%token F_STRCMP

/* check precedence etc! */
%left OP_OR
%left OP_AND
%nonassoc OP_LT OP_LE OP_GT OP_GE OP_EQ OP_NE OP_MATCH
%left OP_PLUS OP_MINUS
%left OP_MUL OP_DIV OP_MOD
%left UMINUS OP_NOT

%type <v_exp> a_expr
%type <v_exp> b_expr
%type <v_exp> s_expr
%type <v_num> CONST_INT
%type <v_id> ID
%type <v_id> STR
%type <v_id> ID_BOOL
%type <v_id> ID_STRING
%type <v_id> ID_INT


%%

start:	b_expr		{ lpctx_P lpctx;
			lpctx = (lpctx_P)ctx;
			lpctx->lpc_rooti = $1; }
	;

b_expr:	a_expr OP_LT a_expr	{ $$ = crt_exp_2(ctx, OP_LT, $1, $3); }
	| a_expr OP_GT a_expr	{ $$ = crt_exp_2(ctx, OP_GT, $1, $3); }
	| a_expr OP_LE a_expr	{ $$ = crt_exp_2(ctx, OP_LE, $1, $3); }
	| a_expr OP_GE a_expr	{ $$ = crt_exp_2(ctx, OP_GE, $1, $3); }
	| a_expr OP_EQ a_expr	{ $$ = crt_exp_2(ctx, OP_EQ, $1, $3); }
	| a_expr OP_NE a_expr	{ $$ = crt_exp_2(ctx, OP_NE, $1, $3); }
	| b_expr OP_AND b_expr	{ $$ = crt_exp_2(ctx, OP_AND, $1, $3); }
	| b_expr OP_OR b_expr	{ $$ = crt_exp_2(ctx, OP_OR, $1, $3); }
	| s_expr OP_MATCH s_expr	{ $$ = crt_exp_2(ctx, OP_MATCH, $1, $3); }
	| F_MATCH O_PARENTHESIS s_expr COMMA s_expr C_PARENTHESIS	{
		$$ = crt_exp_2(ctx, F_MATCH, $3, $5); }
	| OP_NOT b_expr	{ $$ = crt_exp_1(ctx, OP_NOT, $2); }
	| O_PARENTHESIS b_expr C_PARENTHESIS			{ $$ = $2; }
	| ID_BOOL		{ $$ = crt_id(ctx, &($1), vt_bool); }
	;

a_expr:	CONST_INT		{ $$ = crt_int(ctx, $1); }
	| ID_INT		{ $$ = crt_id(ctx, &($1), vt_int); }
	| a_expr OP_PLUS a_expr		{ $$ = crt_exp_2(ctx, OP_PLUS, $1, $3); }
	| a_expr OP_MINUS a_expr	{ $$ = crt_exp_2(ctx, OP_MINUS, $1, $3); }
	| a_expr OP_MUL a_expr		{ $$ = crt_exp_2(ctx, OP_MUL, $1, $3); }
	| a_expr OP_DIV a_expr		{ $$ = crt_exp_2(ctx, OP_DIV, $1, $3); }
	| a_expr OP_MOD a_expr		{ $$ = crt_exp_2(ctx, OP_MOD, $1, $3); }
	| OP_MINUS a_expr %prec UMINUS 	{ $$ = crt_exp_1(ctx, UMINUS, $2); }
	| O_PARENTHESIS a_expr C_PARENTHESIS			{ $$ = $2; }
	| F_STRCMP O_PARENTHESIS s_expr COMMA s_expr C_PARENTHESIS	{
		$$ = crt_exp_2(ctx, F_STRCMP, $3, $5); }
	| F_STRCASECMP O_PARENTHESIS s_expr COMMA s_expr C_PARENTHESIS	{
		$$ = crt_exp_2(ctx, F_STRCASECMP, $3, $5); }
	;

s_expr:	STR		{ $$ = crt_str(ctx, &($1)); }
	| ID_STRING	{ $$ = crt_id(ctx, &($1), vt_str); }
	;

%%

/* should we use re2c to generate a lexer? */

#define STR_QUOTE	'"'

/* note: some of this code relies on a NUL terminated input string! (still?) */
int
yylex(YYSTYPE *lvalp, void *ctx)
{
	int r;
	const char *str;
	char *endptr;
	lpctx_P lpctx;

	assert(ctx != NULL);
	lpctx = (lpctx_P)ctx;
	if (lpctx->lpc_pos > lpctx->lpc_text_n)
		return 0;
	str = lpctx->lpc_text;
	assert(str != NULL);
	str += lpctx->lpc_pos;

	while (lpctx->lpc_pos < lpctx->lpc_text_n && isspace(*str)) {
		++str;
		++lpctx->lpc_pos;
	}
	if (lpctx->lpc_pos >= lpctx->lpc_text_n)
		return 0;

if (YY_Verbose > 0)
sm_io_fprintf(smioerr, "yylex: input=\"%s\"\n", str);
	if (isdigit(*str)) {
		r = CONST_INT;

		/* use memtou32? */
		lvalp->v_num = strtol(str, &endptr, 10);
		lpctx->lpc_pos += endptr - str;
	}
	else if (isalpha(*str)) {
		uint idx;

		endptr = (char *)lpctx->lpc_text + lpctx->lpc_text_n;
		r = chk_name(lpctx ,str, &endptr, &idx);
		if (r == vt_none)
			return -1;
		if (vt_func == r)
			r = functab[idx].f_opcode;
		else {
			lvalp->v_id.id_str = str;
			lvalp->v_id.id_len = endptr - str;
			lvalp->v_id.id_idx = idx;
			switch (r) {
				case vt_id:		r = ID; break;
				case vt_str:	r = ID_STRING; break;
				case vt_bool:	r = ID_BOOL; break;
				case vt_int:	r = ID_INT; break;
				default: return -1;
			}
		}
if (YY_Verbose > 0)
sm_io_fprintf(smioerr, "yylex: id=\"%s\", rest=\"%s\", diff=%d\n", str, endptr, (int)(endptr-str));
		lpctx->lpc_pos += endptr - str;
	}
	else if (*str == STR_QUOTE) {
		char *wptr, *rptr, *endptr;

		r = STR;
		endptr = (char *)lpctx->lpc_text + lpctx->lpc_text_n;
		++str;	/* skip over opening quote */

		for (rptr = wptr = (char *)str; rptr < endptr; ) {
			if (*rptr == '\\') {
				int err;

				++rptr;
				err = sm_conf_backslash(&wptr, &rptr, endptr);
				if (err != 0)
					return -1;
				continue;
			}
			if (*rptr == STR_QUOTE)
				break;
			*wptr++ = *rptr++;
		}
		if (*rptr != STR_QUOTE)
			return -1;
		*rptr = '\0';
		++rptr;
		lvalp->v_id.id_str = str;
		lvalp->v_id.id_len = wptr - str;
		lpctx->lpc_pos += rptr - str + 1;
if (YY_Verbose > 0)
sm_io_fprintf(smioerr, "yylex: str=\"%.*s\", len=%u, rest=\"%s\"\n", (int) lvalp->v_id.id_len, lvalp->v_id.id_str, lvalp->v_id.id_len, rptr);
	} else {
		switch (*str) {
		  case ',': r = COMMA ; break;
		  case '(': r = O_PARENTHESIS ; break;
		  case ')': r = C_PARENTHESIS ; break;
		  case '+': r = OP_PLUS; break;
		  case '-': r = OP_MINUS; break;
		  case '*': r = OP_MUL; break;
		  case '/': r = OP_DIV; break;
		  case '%': r = OP_MOD; break;
		  /* case '~': r = OP_MATCH; break; */
		  case '<':
			if (str[1] != '=') { r = OP_LT; break; }
			r = OP_LE;
			++lpctx->lpc_pos;
			break;

		  case '>':
			if (str[1] != '=') { r = OP_GT; break; }
			r = OP_GE;
			++lpctx->lpc_pos;
			break;

		  case '=':
			if (str[1] != '=' && str[1] != '~') {
				/* SYNTAX ERROR */
				r = 0;
				break;
			}
			r = (str[1] == '~') ? OP_MATCH : OP_EQ;
			++lpctx->lpc_pos;
			break;

		  case '!':
			if (str[1] != '=') { r = OP_NOT; break; }
			r = OP_NE;
			++lpctx->lpc_pos;
			break;

		  case '&':
			if (str[1] != '&') { /* SYNTAX ERROR */ r = 0; break; }
			r = OP_AND;
			++lpctx->lpc_pos;
			break;

		  case '|':
			if (str[1] != '|') { /* SYNTAX ERROR */ r = 0; break; }
			r = OP_OR;
			++lpctx->lpc_pos;
			break;

		  default: r = 0;
		}
		++lpctx->lpc_pos;
	}
if (YY_Verbose > 0)
sm_io_fprintf(smioerr, "yylex=%d, pos=%d, rest=\"%s\"\n", r, lpctx->lpc_pos, lpctx->lpc_text + lpctx->lpc_pos);
	return r;
}
