/*
 * Copyright (c) 2011 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: err2ex.c,v 1.5 2013/12/17 15:31:18 ca Exp $")

#include "sm/error.h"
#include "sm/bdb.h"
#include "smerr2txt.h"
#include "sm/sysexits.h"

/*
**  ERR2EX_DFLT -- "convert" an error code to an exit status.
**
**	Parameters:
**		err -- error code
**
**	Returns:
**		EX_* value
*/

static int
err2ex_dflt(sm_error_T err)
{
	if (sm_is_success(err))
		return EX_OK;
	if (sm_is_temp_err(err))
		return EX_TEMPFAIL;
	if (sm_is_perm_err(err))
		return EX_UNAVAILABLE;
	return EX_SOFTWARE;
}

/*
**  FINDERR -- find error code in table, return rc
**
**	Parameters:
**		err -- error code
**
**	Returns:
**		EX_* value or <0 for not found/not available
*/

static int
finderr(int ecode, e2s_P table)
{
	int i;

	i = 0;
	while (table[i].e2s_code != 0 || table[i].e2s_str != NULL) {
		if (table[i].e2s_code == ecode)
			return table[i].e2s_rc;
		if (sm_error_value(table[i].e2s_code) == sm_error_value(ecode))
			return table[i].e2s_rc;
		++i;
	}
	return -2;
}

/*
**  ERR2EX -- "convert" an error code to an exit status.
**
**	Parameters:
**		err -- error code
**
**	Returns:
**		EX_* value
*/

int
err2ex(sm_error_T err)
{
	int eno, r;
	char *errtxt;

	eno = sm_error_value(err);
	errtxt = NULL;
	r = SM_SUCCESS;

	if (sm_is_ex_error(err))
		return eno;

	if (eno > 0 && eno < sm_sys_nerr)
		return err2ex_dflt(err);
	if (SM_IS_BDB_ERR(err))
		r = finderr(eno, e_bdb4_2);
	else
		r = finderr(eno, e_error);
	if (r >= 0)
		return r;
	return err2ex_dflt(err);
}
