/*
 * Copyright (c) 2010 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: extenhsc.c,v 1.2 2011/03/03 16:32:13 ca Exp $")
#include "sm/assert.h"
#include "sm/ctype.h"
#include "sm/enhsc.h"

/*
**  EXTENHSC -- extract enhanced status code
**
**	Parameters:
**		s -- string with possible enhanced status code.
**		delim -- delim for enhanced status code.
**		enhsc -- caller provided storage for enhanced status code.
**
**	Returns:
**		0  -- no enhanced status code.
**		>4 -- length of enhanced status code.
*/

int
extenhsc(const char *s, int delim, char *enhsc)
{
	uint h, l;

	if (NULL == s)
		return 0;
	SM_REQUIRE(enhsc != NULL);
	if (s[0] != '2' && s[0] != '4' && s[0] != '5')
		return 0;
	enhsc[0] = s[0];
	if (s[1] != '.')
		return 0;
	enhsc[1] = s[1];
	l = 2;
	for (h = 0; h < 3 && ISDIGIT(s[l + h]); h++)
		enhsc[l + h] = s[l + h];
	if (h == 0 || s[l + h] != '.')
		return 0;
	enhsc[l + h] = s[l + h];
	l += h + 1;
	for (h = 0; h < 3 && ISDIGIT(s[l + h]); h++)
		enhsc[l + h] = s[l + h];
	if (h == 0 || s[l + h] != delim)
		return 0;
	enhsc[l + h] = '\0';

	return l + h;
}
