/*
 */

#include "sm/generic.h"
#include "sm/binsearch.h"
#include "sm/assert.h"
#include "sm/test.h"

static sm_identry_T idtab[] = {
	  { "abc" }
	, { "defg" }
	, { "thisisme" }
	, { "xzy" }
	, { "xzy0" }
	, { "xzyy" }
};

static int Verbose;

sm_ret_T
sm_binsearchn(sm_identry_P idtab, int sz, const char *id, int len, int *pidx)
{
	int cmp, low, mid, high;

	SM_REQUIRE(id != NULL);
	SM_REQUIRE(pidx != NULL);
	SM_REQUIRE(idtab != NULL);
	SM_REQUIRE(sz > 0);

	low = 0;
	high = sz - 1;
	while (low <= high) {
		mid = (low + high) / 2;
		cmp = strncmp(id, idtab[mid].idt_id, len);
		if (cmp == 0)
			cmp = len - strlen(idtab[mid].idt_id);
		if (cmp == 0) {
			*pidx = mid;
			return SM_SUCCESS;
		}
		else if (cmp < 0)
			high = mid - 1;
		else if (cmp > 0)
			low = mid + 1;
	}
	return sm_err_perm(SM_E_NOTFOUND);
}

void
testbl(void)
{
	int i, sz, idx;

	sz = SM_ARRAY_SIZE(idtab);
	for (i = 0; i < sz; i++) {
		SM_TEST(sm_binsearchn(idtab, sz, idtab[i].idt_id, strlen(idtab[i].idt_id), &idx) == SM_SUCCESS);
		SM_TEST(idx == i);
	}
	SM_TEST(sm_binsearch(idtab, sz, "not", &idx) != SM_SUCCESS);
}

void
testb(void)
{
	int i, sz, idx;

	sz = SM_ARRAY_SIZE(idtab);
	for (i = 0; i < sz; i++) {
		SM_TEST(sm_binsearch(idtab, sz, idtab[i].idt_id, &idx) == SM_SUCCESS);
		SM_TEST(idx == i);
	}
	SM_TEST(sm_binsearch(idtab, sz, "not", &idx) != SM_SUCCESS);
}


int
main(int argc, char *argv[])
{
	int c;

	while ((c = getopt(argc, argv, "V")) != -1) {
		switch (c) {
		  case 'V':
			++Verbose;
			break;
		  default:
			return(1);
		}
	}
	sm_test_begin(argc, argv, "test binsearch");
	testb();
	testbl();
	return sm_test_end();
}
