/*
 * Copyright (c) 2010 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-bht-2.c,v 1.8 2013/10/23 22:13:55 ca Exp $")

#include "sm/error.h"
#include "sm/heap.h"
#include "sm/memops.h"
#include "sm/test.h"
#include "sm/maps.h"
#include "sm/mapc.h"
#include "sm/map.h"
#include "sm/mapclasses.h"
#include "sm/bdb.h"

#include "sm/io.h"

int Verbose = 0;


/* outgoing (smtpc) connection limits */
struct sc_cl_S {
	char		*sccl_rcpt_domain;
	uint32_t	 sccl_max_tas_se;
};
typedef struct sc_cl_S sc_cl_T, *sc_cl_P;

/* see below: <= 1 will not be used for lookups */
sc_cl_T cl_t[] = {
	 { "example.com", 41 }
	,{ "example.org", 2 }
	,{ "example.net", 23 }
	,{ "example.double", 1 }
	,{ "example.double", 0 }
};

#define MAPC_TYPE	"bhtable"
#define MAPC_NAME	"bht0"
#define MAPC_FILE	"./bht0.db"

static sm_ret_T
cl_map_init(sm_map_P map)
{
	uint n, u;
	sm_ret_T ret;
	sm_str_T key, data;
	size_t datalen;

	ret = SM_SUCCESS;
	datalen = sizeof(void *);
	n = SM_ARRAY_SIZE(cl_t);
	for (u = 0; u < n; u++) {
		const char *domain;
		size_t domainlen;

		domain = cl_t[u].sccl_rcpt_domain;
		domainlen = strlen(domain);

		sm_str_assign(key, NULL, (uchar *)domain, domainlen, domainlen + 2);
		sm_str_assign(data, NULL, (uchar*)&(cl_t[u]), datalen, datalen);

		ret = sm_map_add(map, &key, &data, SMMAP_AFL_UNIQUE);
/* sm_io_fprintf(smioerr, "key=%S, rhs=%p\n", &key, &(cl_t[u])); */
		if (cl_t[u].sccl_max_tas_se == 0) {
			if (sm_is_err(ret)) {
				ret = SM_SUCCESS;
				continue;
			}
			ret = sm_err_perm(EINVAL);	/* better error code? */
		}
		if (sm_is_err(ret))
			break;
	}

	return ret;
}

static void
testh(void)
{
	uint n, u;
	sm_ret_T ret;
	sm_maps_P maps;
	sm_map_P map;
	sm_cstr_P mtype, mname;
	sm_str_P lhs, rhs;
	sm_str_T sc_rhs;
	const char *domain;
	sc_cl_P sc_cl;

	maps = NULL;
	mtype = mname = NULL;
	lhs = rhs = NULL;

	ret = sm_maps_init(&maps);
	SM_TEST(maps != NULL);
	if (maps == NULL)
		return;
	SM_TEST(sm_is_success(ret));

	mtype = sm_cstr_scpyn0((const uchar *)MAPC_TYPE, strlen(MAPC_TYPE));
	SM_TEST(mtype != NULL);
	if (mtype == NULL)
		goto error;

	mname = sm_cstr_scpyn0((const uchar *)MAPC_NAME, strlen(MAPC_NAME));
	SM_TEST(mname != NULL);
	if (mname == NULL)
		goto error;

	lhs = sm_str_new(NULL, 256, 1024);
	SM_TEST(lhs != NULL);
	if (lhs == NULL)
		goto error;

	rhs = sm_str_new(NULL, sizeof(void *), sizeof(void *) + 2);
	SM_TEST(rhs != NULL);
	if (rhs == NULL)
		goto error;

	ret = sm_bht_class_create(maps);
	SM_TEST(sm_is_success(ret));

	map = NULL;
	ret = sm_map_create(maps, mtype, 0, &map);
	SM_TEST(sm_is_success(ret));
	if (!sm_is_success(ret))
		goto error;

	ret = sm_map_open(maps, mname, mtype, 0, NULL
			, SMAP_MODE_RDONLY, &map
			, SMPO_HASH_NELEM, 21, SMPO_ELEM_MAX, 0
			, SMPO_INIT_CBF, cl_map_init, SMPO_INIT_CBC, map
			, SMPO_END);
	SM_TEST_E(sm_is_success(ret));

	/* perform some operations ... */

	n = SM_ARRAY_SIZE(cl_t);
	sm_str_assign(sc_rhs, NULL, NULL, sizeof(void *), sizeof(void *));
	for (u = 0; u < n; u++) {

		if (cl_t[u].sccl_max_tas_se <= 1)
			continue;

		domain = cl_t[u].sccl_rcpt_domain;
		sm_str_clr(lhs);
		ret = sm_str_scat(lhs, domain);
		SM_TEST_E(sm_is_success(ret));
		ret = sm_map_lookup(map, SMMAP_LFL_USE_PTR, lhs, &sc_rhs);
		SM_TEST_E(sm_is_success(ret));
		sc_cl = (sc_cl_P)sm_str_data(&sc_rhs);
		SM_TEST_E(sc_cl == &cl_t[u]);
		SM_TEST(sm_streq(sc_cl->sccl_rcpt_domain, cl_t[u].sccl_rcpt_domain));

		ret = sm_map_lookup_domain(map, (sm_rdstr_P) lhs, NULL, SMMAP_LFL_USE_PTR, &sc_rhs);
		SM_TEST_E(sm_is_success(ret));
	}

	ret = sm_map_close(map, 0);
	SM_TEST(sm_is_success(ret));

	ret = sm_maps_term(maps);
	SM_TEST(sm_is_success(ret));
	SM_CSTR_FREE(mtype);
	SM_CSTR_FREE(mname);
	SM_STR_FREE(lhs);
	SM_STR_FREE(rhs);
	return;

  error:
	sm_maps_term(maps);
}


int
main(int argc, char *argv[])
{
/*
	int c;

	while ((c = getopt(argc, argv, "H:s:V")) != -1) {
		switch (c) {
		  default:
			return 1;
		}
	}
*/
	sm_test_begin(argc, argv, "test bht map 2");
	testh();
	return sm_test_end();
}
